#vider la mmoire
rm(list=ls())

#librairie
library(xlsx)

#charger
donnees <- read.xlsx("Low_Birth_Weight_Data.xlsx",sheetIndex = 1, header=TRUE)

#premieres lignes
print(head(donnees))

#description
str(donnees)

#modle : LOW vs. l'ensemble des variables explicatives
modele <- glm(LOW ~ ., data = donnees, family = binomial)

#rsultats dtaills - 3 coefficients (de variables) prsentent une p-value < 0.05
print(summary(modele))

#probabilits d'affectation
scores <- predict(modele, newdata = donnees, type = "response")
print(summary(scores))

#***********
# courbe ROC
#***********


#coder la cibe en 0/1
y <- ifelse(donnees$LOW == "y",1,0)

#vrification
print(table(donnees$LOW,y))

#rcupration du nombre de positifs
npos <- sum(y)
print(npos)

#nombre de ngatifs
nneg <- sum(1-y)
print(nneg)

#crer un data frame avec y et scores
df <- data.frame(y,scores)
print(head(df))

#trier le data frame avec les scores decroissants
dfSorted <- df[order(scores,decreasing=TRUE),]

#premires lignes
print(head(dfSorted))

#dernires lignes
print(tail(dfSorted))

#taux de faux positifs
tfp <- cumsum(1-dfSorted$y)/nneg
print(head(tfp))

#taux de vrais positifs
tvp <- cumsum(dfSorted$y)/npos
print(head(tvp))

#graphique - courbe ROC
plot(tfp,tvp,main="Courbe ROC",type="l",col="blue")
abline(a=0,b=1)

#utilisation de la librairie ROCR
library(ROCR)

#construire un objet prediction avec les scores et la var. cible
pred <- ROCR::prediction(scores,donnees$LOW)
print(pred)

#objet perfromance mesure
graphs_roc <- ROCR::performance(pred,measure="tpr",x.measure="fpr")
print(graphs_roc)

#graphique courbe ROC
ROCR::plot(graphs_roc,xlab="TFP",ylab="TFP",col="darkblue")
abline(a=0,b=1)

#****************
# calcul de l'AUC
#****************

#calcul de l'AUC
#rangs - tout simplement inverse du numro pour la base trie
rk <- seq(length(y),1,-1)
print(rk)

#somme des rangs des positifs
s_plus <- sum(rk * dfSorted$y)
print(s_plus)

#stat de Mann-Whitney
U_plus <- s_plus - 0.5*(npos*(npos+1))
print(U_plus)

#auc
auc <- U_plus/(npos*nneg)
print(auc)

#avec la librairie ROCR - objet performance
auc_roc <- ROCR::performance(pred,measure="auc")
print(auc_roc)

#**********************************
#calcul en leave-one-out des scores
#**********************************

#vecteur prparatoire des rsultats
scores_lvo <- rep(0,length(y))

#boucle de calcul
for (i in 1:length(y)){
  #modlisation
  temp_modele <- glm(LOW ~ ., data = donnees[-i,], family = binomial)
  #prdiction
  scores_lvo[i] <- predict(temp_modele, newdata = donnees[i,], type = "response")
}

#summary des scores
print(summary(scores_lvo))

#calcul de prediction au sens de ROCR
pred_lvo <- ROCR::prediction(scores_lvo,donnees$LOW)

#performance au sens de ROCR
graphs_roc_lvo <- ROCR::performance(pred_lvo,measure="tpr",x.measure="fpr")

#courbe ROC en LVO en vert
ROCR::plot(graphs_roc_lvo,xlab="TFP",ylab="TFP",col="green", main="Courbe ROC")
abline(a=0,b=1)

#rajouter la courbe en resubstitution
#qui est effectivement survalue
lines(tfp,tvp,col="blue")

#ajouter une lgende
legend(0,0.95,legend=c("Resubstitution","Leave-one-out"),col=c("blue","green"),lty=1:2)

#calcul de l'AUC
#on la valeur corrige, plus raliste
#plus conforme aux vritables performances du modle dans la population
auc_roc_lvo <- ROCR::performance(pred_lvo,measure="auc")
print(auc_roc_lvo)
